/**
Copyright © 2014 Marko Niinimaki. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**/
package org.natto;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

/**
 * Implements a simple "should I execute the task or wait".
 * @author man@cern.ch
 */
public class PoliteScheduler {

    private static int WAIT_ROUNDS = 5;
    private static HashMap jobsWaiting = new HashMap();
    /**
     * Checks if this host is the best among those that can
     * execute the task.
     * @param fileName name of the job description file
     * @param myIname the id file of this host
     * @param allInfos info hashes
     * @return true if I am the best.
     */
    public static boolean yesIshouldTakeit(String fileName,
                                           String myIname, HashMap allInfos) {
        //sum up stuff in up-to-date infofiles
        long myPower = 0;
        long bestPower = 0;
        try {
            Properties myOwnInfo = (Properties) allInfos.get(myIname);
            myPower = getRating(myOwnInfo);
        } catch (Exception e) { return false; }
        Set<Object> keySet = allInfos.keySet();
        Iterator myIter = keySet.iterator();
        while (myIter.hasNext()) {
            String currentKey = (String) myIter.next();
            Properties infoProps = (Properties) allInfos.get(currentKey);
            long thePower = getRating(infoProps);
            if (thePower > bestPower) { bestPower = thePower; }
        }
        //test if we have the best power or we have waited enough
        if (myPower == bestPower) {
            System.out.println("Me, best available " + bestPower);
            return true;
        } else {
            System.out.println("No, I am not the best host, I'll wait.");
        }
        if (!(jobsWaiting.containsKey(fileName))) {
            jobsWaiting.put(fileName, 0);
        }
        //increase the wait for this job by 1
        int jobHasWaited = (Integer) jobsWaiting.get(fileName);
        jobHasWaited++;
        jobsWaiting.put(fileName, jobHasWaited);
        if (jobHasWaited == WAIT_ROUNDS)  {
            jobsWaiting.remove(fileName);
            return true;
        }
        return false;
    }

    /**
     * Get a rating for this host.
     * @param thisInfo info properties of the host
     * @return rating
     */
    static long getRating(Properties thisInfo) {
        long rating;
        String tmpStr = thisInfo.getProperty("cores", "1");
        int cores = Integer.parseInt(tmpStr);
        tmpStr = thisInfo.getProperty("mhz", "1");
        long mHz = Long.parseLong(tmpStr);
        tmpStr = thisInfo.getProperty("mmem", "1");
        long mMem = Long.parseLong(tmpStr);
        tmpStr = thisInfo.getProperty("tasks", "0");
        int numRunningTasks = Integer.parseInt(tmpStr);
        rating = (cores - numRunningTasks) * mHz + mMem;
        return rating;
    }

}
