/**
Copyright © 2014 Marko Niinimaki. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**/
package org.natto.swingGUI;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/** For submitting jobs by GUI */
public class SubmitGui extends JFrame implements ActionListener {
    /** menu */
    private JMenu helpMenu;
    /** upload button 1 and its text*/
    private JButton fileButton1;
    private JLabel f1Text;
    /** upload button 2 and its text*/
    private JButton fileButton2;
    private JLabel f2Text;
    /** files to be uploaded */
    private File file1 = null;
    private File zipFile = null; //in case file1 is an executable in a zip
    private File file2 = null;
    //a text field to contain optional command line
    private JTextField cLine;
    /** working directory, get from constructor */
    private String workDir = "";
    /** info file hash, set in constructor */
    private HashMap infoHash = null;
    /** obvious */
    private JButton cancelButton;
    private JButton submitButton;
    /** optional: submit to this host */
    private String submitTarget = null;

    /** Constructor.
     * @param myDir the directory where the executor runs.
     * @param infos info file hash map.
     * @param targetHost if not null/empty: target of submission
     */
   public SubmitGui(String myDir, HashMap infos, String targetHost) {
       this.setTitle("Submit");
       if ((targetHost != null) && (!targetHost.equals(""))) {
           this.setTitle("Submit to " + targetHost);
           submitTarget = targetHost;
       }
       workDir = myDir;
       infoHash = infos;
       setupGUI();
       setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }

   /** Make buttons and labels.
    */
   void setupGUI() {
        setLayout(new BorderLayout(10, 10));
	JMenuBar menubar = new JMenuBar();
	helpMenu = new JMenu("Help and examples");
	menubar.add(helpMenu);
        setJMenuBar(menubar);
	myMenuListener mml = new  myMenuListener();
        helpMenu.addMenuListener(mml);
        JPanel textAndButton1 = new JPanel();
        textAndButton1.setLayout(new BorderLayout());
	f1Text = new JLabel();
        String supportedExt = org.natto.Infofiles.getSupportedExt(infoHash);
        String uploadInst = "<html>Upload file 1 (compulsory).<br/>"
                       + "This file can be a binary or<br/>"
                       + "a script with a supported type:<br/>"
                       + supportedExt + "</html>";
        f1Text.setText(uploadInst);
	textAndButton1.add(f1Text, BorderLayout.WEST);

     	fileButton1 = new JButton();
	fileButton1.setText("File #1");
	fileButton1.setPreferredSize(new Dimension(90, 20));
	textAndButton1.add(fileButton1, BorderLayout.EAST);
        fileButton1.addActionListener(this);
        //add this line in the grid
	getContentPane().add(textAndButton1, BorderLayout.PAGE_START);

        JPanel textAndButton2 = new JPanel();
        textAndButton2.setLayout(new BorderLayout());
	f2Text = new JLabel();
	f2Text.setText("<html>Upload file 2 (optional).<br/>"
                       + "This file can be a data file for<br/>"
                       + "file 1. If you need more than one<br/>"
                       + "data files, please zip them and<br/>"
                       + "upload the zip file.<br/>"
                       + "If your data files are already<br/>"
                       + "in this directory, you do not need<br/>"
                       + "to upload them.</html>");
	textAndButton2.add(f2Text, BorderLayout.WEST);

	fileButton2 = new JButton();
	fileButton2.setPreferredSize(new Dimension(90, 20));
	fileButton2.setText("File #2");
	textAndButton2.add(fileButton2, BorderLayout.EAST);
        fileButton2.addActionListener(this);
	getContentPane().add(textAndButton2, BorderLayout.CENTER);

        //panel to contain the command line panel and submit/cancel panel
        JPanel clAndButtons = new JPanel();
        clAndButtons.setLayout(new BorderLayout());
        //add the command line and instruction panel
        JPanel clPanel = new JPanel();
        clPanel.setLayout(new BorderLayout());
	JLabel clText = new JLabel();
	clText.setText("Command line extra arguments (optional)");
	clPanel.add(clText, BorderLayout.NORTH);

        //note: cLine has been introduced in the class
	cLine = new JTextField(30);
        //cLine.setPreferredSize(new Dimension(100, 20));
	cLine.setEditable(true);
	clPanel.add(cLine, BorderLayout.SOUTH);

        clAndButtons.add(clPanel, BorderLayout.NORTH);

        JPanel scPanel = new JPanel();
        //submit and cancel buttons. NB: submit is initially disabled.
	submitButton = new JButton();
	submitButton.setText("Submit");
        submitButton.setEnabled(false);
	scPanel.add(submitButton, BorderLayout.EAST);
        submitButton.addActionListener(this);

	cancelButton = new JButton();
	cancelButton.setText("Cancel");
	scPanel.add(cancelButton, BorderLayout.WEST);
        cancelButton.addActionListener(this);
        clAndButtons.add(scPanel, BorderLayout.SOUTH);

	getContentPane().add(clAndButtons, BorderLayout.PAGE_END);

        pack();
	setVisible(true);
	setResizable(true);
    }

    public void actionPerformed(ActionEvent e) {
        //which button/menu is this?
        if (e.getSource() == helpMenu) {
	    System.out.println("help");
	}
        if (e.getSource() == submitButton) {
	    //System.out.println("submit");
	    //get the optional command line text, if the user entered something
	    String cLineExtra = "";
	    if (cLine != null) {
		cLineExtra = cLine.getText();
	    }
            submitJob(cLineExtra, submitTarget);
	}
        if ((e.getSource() == fileButton1) || (e.getSource() == fileButton2)) {
	    JFileChooser fc = new JFileChooser(workDir);
	    int rVal = fc.showOpenDialog(SubmitGui.this);
	    if (rVal == JFileChooser.APPROVE_OPTION) {
                File selFile = fc.getSelectedFile();
                String fullFname = selFile.getAbsolutePath();
                if (e.getSource() == fileButton1) {
                    //check that the file has an extension
                    //that we like or is ELF
                    //or is a zip file that contains a program
                    //that we support
                    String compLaint = fileComplain(selFile);
                    if (!("".equals(compLaint))) {
			JOptionPane.showMessageDialog(null,
                           "Sorry, unsupported extension.");
			return;
		    } else {
                        file1 = selFile;
                        //was this a .zip with some stuff inside
                        if (fullFname.toLowerCase().endsWith(".zip")) {
                            String inZipNm = getFileSupportedInZip(fullFname);
                            zipFile = selFile;
                            file1 = new File(workDir + "/" + inZipNm);
                        }
		    }
		} else { file2 = selFile; }

                //put the file in the working directory
                //System.out.println("Copying " + fname + " to " + workDir);
		org.natto.StaticTools.copyFileIfNeeded(selFile, workDir);
                submitButton.setEnabled(true);
	    }
	}
        if (e.getSource() == cancelButton) {
	    System.out.println("cancel");
	    setVisible(false);
	    dispose();
	}
    }

    String fileComplain(File myFile) {
        //check if the file has an acceptable ext or is ELF
        //or has nice stuff in zip
	if (org.natto.StaticTools.fileIsELF(myFile)) {
	    return "";
	}
        //get the file extension
        String fname = myFile.getName();
        String myExt = org.natto.StaticTools.getFileExt(fname);
        if (myExt == null) { return "File has no extension."; }
        if (".zip".equals(myExt.toLowerCase())) {
            String fullName = myFile.getAbsolutePath();
            if (!(getFileSupportedInZip(fullName).equals(""))) {
                return "";
            }
        }
        String myPrg = org.natto.Infofiles.getPrgForExt(myExt, infoHash);
        if (myPrg == null) {
	    return "Could not find the program for extension " + myExt;
	}
	return "";
    }

    /**
     * Returns a file name of a file that is in zip and has a supported ext.
     * Example: foo.sh in /users/me/foo.zip
     * @param fullZipName zip file name, full path.
     * @return file name or "" if none.
     */
    String getFileSupportedInZip(String fullZipName) {
        File myFile = new File(fullZipName);
        String shortName = myFile.getName();
        //get the list of files from the zip
        String[] filesInZip = org.natto.StaticTools.unZip(
                    fullZipName, true);
        String allExtsStr = org.natto.Infofiles.getSupportedExt(infoHash);
        String[] allExts = allExtsStr.split(" ");
        String myNoExt = shortName.substring(0, shortName.length() - 4);
        //see if the files contain myNoExt + .ext for something
        for (int i=0; i<filesInZip.length; i++) {
            String inZip = filesInZip[i];
            for (int j=0; j<allExts.length; j++) {
                String candFn = myNoExt + allExts[j];
                if (candFn.equals(inZip)) {
                        return inZip;
                }
            }
        }
        return "";
    }

    /** response to submit, fill */
    void submitJob(String clineExtra, String targetHost) {
        org.natto.jobDescription myJd
                = new org.natto.jobDescription(workDir);
        String diags = myJd.writeJobDescription(file1, file2, zipFile,
                                                infoHash, clineExtra,
                                                targetHost);
	JOptionPane.showMessageDialog(null, diags);
	setVisible(false);
	dispose();
    }

    class myMenuListener implements MenuListener {
	public void menuCanceled(MenuEvent e) {}
        public void menuSelected(MenuEvent e) {
	    //System.out.println("sel");
	    try {
		String toPage =
                   "https://dl.dropboxusercontent.com/u/919611/executor.html";
		org.natto.swingGUI.BareBonesBrowserLaunch.openURL(toPage);
	    } catch (Exception ex) { ex.printStackTrace(); }
	}
        public void menuDeselected(MenuEvent e) {}
    }

    public static void nomain(String args[]) {
	String myDir = System.getProperty("user.dir");
	HashMap myHm = org.natto.Infofiles.checkInfoFiles(myDir, "", 300);
	new SubmitGui(myDir, myHm, "");
   }
}
