/**
Copyright © 2014 Marko Niinimaki. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**/
package org.natto.swingGUI;

import javax.swing.DefaultListModel;

/**
 * Show a list of hosts from which the user can select one.
 * @author man@cern.ch
 */
public class hostSelectDialog extends javax.swing.JDialog {

    private String selectAction = "submit";
    private String selectedHost = null;
    /**
     * Creates new form hostSelectDialog.
     * @param parent parent frame, can be null
     * @param modal dialog modality
     * @param hostNames array of host names
     */
    public hostSelectDialog(java.awt.Frame parent, boolean modal,
                            String[] hostNames) {
        super(parent, modal);
        DefaultListModel DLM = new DefaultListModel();
        for (int i = 0; i < hostNames.length; i++) {
            String hName = hostNames[i];
            DLM.addElement(hName);
        }
        initComponents();
        hostList.setModel(DLM);
        hostList.setSelectedIndex(0);
        setVisible(true);
    }

    /** Returms the selected host and action (submit, see).
     * @return host and action
     **/
    public String[] getSelectedHostAndAction() {
        return new String[] {selectedHost, selectAction};
    }
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        hostList = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        instrucTions = new javax.swing.JTextPane();
        jPanel1 = new javax.swing.JPanel();
        seeFilesButton = new javax.swing.JButton();
        submitButton = new javax.swing.JButton();

        setTitle("Select host");
        setMinimumSize(new java.awt.Dimension(60, 50));
        setPreferredSize(new java.awt.Dimension(270, 180));

        hostList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        hostList.setSelectedIndex(1);
        jScrollPane1.setViewportView(hostList);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        instrucTions.setEditable(false);
        instrucTions.setText("Select the host or click here to cancel");
        instrucTions.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                closeOnClick(evt);
            }
        });
        jScrollPane2.setViewportView(instrucTions);

        getContentPane().add(jScrollPane2, java.awt.BorderLayout.PAGE_START);

        jPanel1.setLayout(new java.awt.BorderLayout());

        seeFilesButton.setText("See the files in this host");
        seeFilesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                seeHostAction(evt);
            }
        });
        jPanel1.add(seeFilesButton, java.awt.BorderLayout.CENTER);

        submitButton.setText("Submit to this host");
        submitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                submitButtonActionPerformed(evt);
            }
        });
        jPanel1.add(submitButton, java.awt.BorderLayout.PAGE_START);

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_END);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void closeOnClick(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_closeOnClick
        setVisible(false);
    }//GEN-LAST:event_closeOnClick

    private void submitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_submitButtonActionPerformed
       String myHost = (String)hostList.getSelectedValue();
       //get the value btw parenteses
       int leftParPos = myHost.lastIndexOf("(");
       int rightParPos = myHost.lastIndexOf(")");
       if (leftParPos > -1) {
            if (rightParPos > leftParPos) {
                selectedHost = myHost.substring(leftParPos + 1,
                    rightParPos);
            }
       }
       setVisible(false);
    }//GEN-LAST:event_submitButtonActionPerformed

    private void seeHostAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_seeHostAction
        submitButtonActionPerformed(evt);
        selectAction = "see";
    }//GEN-LAST:event_seeHostAction



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList hostList;
    private javax.swing.JTextPane instrucTions;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JButton seeFilesButton;
    private javax.swing.JButton submitButton;
    // End of variables declaration//GEN-END:variables
}
